/* Automatically generated by po2tbl.sed from mutt.pot.  */

#if HAVE_CONFIG_H
# include <config.h>
#endif

#include "libgettext.h"

const struct _msg_ent _msg_tbl[] = {
  {"", 1},
  {"Exit", 2},
  {"Select", 3},
  {"Help", 4},
  {"You have no aliases!", 5},
  {"Aliases", 6},
  {"Alias as: ", 7},
  {"You already have an alias defined with that name!", 8},
  {"Address: ", 9},
  {"Personal name: ", 10},
  {"[%s = %s] Accept?", 11},
  {"Save to file: ", 12},
  {"Alias added.", 13},
  {"Can't match nametemplate, continue?", 14},
  {"Mailcap compose entry requires %%s", 15},
  {"Error running \"%s\"!", 16},
  {"Failure to open file to parse headers.", 17},
  {"Failure to open file to strip headers.", 18},
  {"No mailcap compose entry for %s, creating empty file.", 19},
  {"Mailcap Edit entry requires %%s", 20},
  {"No mailcap edit entry for %s", 21},
  {"No matching mailcap entry found.  Viewing as text.", 22},
  {"MIME type not defined.  Cannot view attachment.", 23},
  {"Cannot create filter", 24},
  {"Attachment saved.", 25},
  {"Write fault!", 26},
  {"I don't know how to print that!", 27},
  {"Chdir", 28},
  {"Mask", 29},
  {"%s is not a directory.", 30},
  {"Mailboxes [%d]", 31},
  {"Directory [%s], File mask: %s", 32},
  {"No files match the file mask", 33},
  {"Chdir to: ", 34},
  {"Error scanning directory.", 35},
  {"File Mask: ", 36},
  {"Reverse sort by (d)ate, (a)lpha, si(z)e or do(n)'t sort? ", 37},
  {"Sort by (d)ate, (a)lpha, si(z)e or do(n)'t sort? ", 38},
  {"dazn", 39},
  {"New file name: ", 40},
  {"Can't view a directory", 41},
  {"Error trying to view file", 42},
  {"New mail in %s.", 43},
  {"%s: color not supported by term", 44},
  {"%s: no such color", 45},
  {"%s: no such object", 46},
  {"%s: command valid only for index object", 47},
  {"%s: too few arguments", 48},
  {"Missing arguments.", 49},
  {"color: too few arguments", 50},
  {"mono: too few arguments", 51},
  {"%s: no such attribute", 52},
  {"too few arguments", 53},
  {"too many arguments", 54},
  {"default colors not supported", 55},
  {"Invoking PGP...", 56},
  {"Verify PGP signature?", 57},
  {"Could not create temporary file!", 58},
  {"Command: ", 59},
  {"Bounce message to: ", 60},
  {"Bounce tagged messages to: ", 61},
  {"Error parsing address!", 62},
  {"Bounce message to %s", 63},
  {"Bounce messages to %s", 64},
  {"Message bounced.", 65},
  {"Messages bounced.", 66},
  {"Pipe to command: ", 67},
  {"\
Rev-Sort (d)ate/(f)rm/(r)ecv/(s)ubj/t(o)/(t)hread/(u)nsort/si(z)e/s(c)ore?: ", 68},
  {"\
Sort (d)ate/(f)rm/(r)ecv/(s)ubj/t(o)/(t)hread/(u)nsort/si(z)e/s(c)ore?: ", 69},
  {"dfrsotuzc", 70},
  {"Shell command: ", 71},
  {"%s%s to mailbox", 72},
  {"Decode-save", 73},
  {"Decode-copy", 74},
  {"Decrypt-save", 75},
  {"Decrypt-copy", 76},
  {"Save", 77},
  {"Copy", 78},
  {" tagged", 79},
  {"Copying to %s...", 80},
  {"Print message?", 81},
  {"Print tagged messages?", 82},
  {"Message printed", 83},
  {"Messages printed", 84},
  {"Send", 85},
  {"Abort", 86},
  {"Attach file", 87},
  {"Descrip", 88},
  {"Sign, Encrypt", 89},
  {"Encrypt", 90},
  {"Sign", 91},
  {"Clear", 92},
  {" sign as: ", 93},
  {"<default>", 94},
  {"MIC algorithm: ", 95},
  {"\
(e)ncrypt, (s)ign, sign (a)s, (b)oth, select (m)ic algorithm, or (f)orget \
it? ", 96},
  {"esabmf", 97},
  {"Can't open your secret key ring!", 98},
  {"Sign as: ", 99},
  {"An unkown PGP version was defined for signing.", 100},
  {"This doesn't make sense if you don't want to sign the message.", 101},
  {"Unknown MIC algorithm, valid ones are: pgp-md5, pgp-sha1, pgp-rmd160", 102},
  {"%s [#%d] no longer exists!", 103},
  {"%s [#%d] modified. Update encoding?", 104},
  {"===== Attachments =====", 105},
  {"You may not delete the only attachment.", 106},
  {"Compose", 107},
  {"Open mailbox to attach message from", 108},
  {"No messages in that folder.", 109},
  {"Tag the messages you want to attach!", 110},
  {"Unable to attach!", 111},
  {"Invalid encoding.", 112},
  {"Save a copy of this message?", 113},
  {"Rename to: ", 114},
  {"Can't stat: %s", 115},
  {"New file: ", 116},
  {"Content-Type is of the form base/sub", 117},
  {"Unknown Content-Type %s", 118},
  {"Can't create file %s", 119},
  {"What we have here is a failure to make an attachment", 120},
  {"Postpone this message?", 121},
  {"Write message to mailbox", 122},
  {"Writing message to %s ...", 123},
  {"Message written.", 124},
  {"yes", 125},
  {"no", 126},
  {"Exit Mutt?", 127},
  {"unknown error", 128},
  {"Press any key to continue...", 129},
  {" ('?' for list): ", 130},
  {"No mailbox is open.", 131},
  {"There are no messages.", 132},
  {"Mailbox is read-only.", 133},
  {"Function not permitted in attach-message mode.", 134},
  {"Cannot toggle write on a readonly mailbox!", 135},
  {"Changes to folder will be written on folder exit.", 136},
  {"Changes to folder will not be written.", 137},
  {"Quit", 138},
  {"Del", 139},
  {"Undel", 140},
  {"Mail", 141},
  {"Reply", 142},
  {"Group", 143},
  {"Mailbox was externally modified.  Flags may be wrong.", 144},
  {"New mail in this mailbox.", 145},
  {"No tagged messages.", 146},
  {"Jump to message: ", 147},
  {"Argument must be a message number.", 148},
  {"That message is not visible.", 149},
  {"Invalid message number.", 150},
  {"Delete messages matching: ", 151},
  {"No limit pattern is in effect.", 152},
  {"Limit: %s", 153},
  {"Limit to messages matching: ", 154},
  {"Quit Mutt?", 155},
  {"Tag messages matching: ", 156},
  {"Undelete messages matching: ", 157},
  {"Untag messages matching: ", 158},
  {"Open mailbox in read-only mode", 159},
  {"Open mailbox", 160},
  {"%s is not a mailbox.", 161},
  {"Exit Mutt without saving?", 162},
  {"You are on the last message.", 163},
  {"No undeleted messages.", 164},
  {"You are on the first message.", 165},
  {"Search wrapped to top.", 166},
  {"Search wrapped to bottom.", 167},
  {"No new messages", 168},
  {"No unread messages", 169},
  {" in this limited view", 170},
  {"No more threads.", 171},
  {"You are on the first thread.", 172},
  {"Thread contains unread messages.", 173},
  {"Threading is not enabled.", 174},
  {"\
~~\t\tinsert a line begining with a single ~\n\
~b users\tadd users to the Bcc: field\n\
~c users\tadd users to the Cc: field\n\
~f messages\tinclude messages\n\
~F messages\tsame as ~f, except also include headers\n\
~h\t\tedit the message header\n\
~m messages\tinclude and quote messages\n\
~M messages\tsame as ~m, except include headers\n\
~p\t\tprint the message\n\
~q\t\twrite file and quit editor\n\
~r file\t\tread a file into the editor\n\
~t users\tadd users to the To: field\n\
~u\t\trecall the previous line\n\
~v\t\tedit message with the $visual editor\n\
~w file\t\twrite message to file\n\
~x\t\tabort changes and quit editor\n\
~?\t\tthis message\n\
.\t\ton a line by itself ends input\n", 175},
  {"%d: invalid message number.\n", 176},
  {"(End message with a . on a line by itself)\n", 177},
  {"No mailbox.\n", 178},
  {"Message contains:\n", 179},
  {"(continue)\n", 180},
  {"missing filename.\n", 181},
  {"No lines in message.\n", 182},
  {"%s: unknown editor command (~? for help)\n", 183},
  {"Set flag", 184},
  {"Clear flag", 185},
  {"[-- Error: unexpected end of file! --]\n", 186},
  {"[-- Error:  Could not display any parts of Multipart/Alternative! --]\n", 187},
  {"[-- Attachment #%d", 188},
  {"[-- Type: %s/%s, Encoding: %s, Size: %s --]\n", 189},
  {"[-- Autoview using %s --]\n", 190},
  {"Invoking autoview command: %s", 191},
  {"[-- Autoview stderr of %s --]\n", 192},
  {"[-- Error: message/external-body has no access-type parameter --]\n", 193},
  {"[-- This %s/%s attachment ", 194},
  {"(size %s bytes) ", 195},
  {"has been deleted --]\n", 196},
  {"[-- on %s --]\n", 197},
  {"\
[-- This %s/%s attachment is not included, --]\n\
[-- and the indicated external source has --]\n\
[-- expired. --]\n", 198},
  {"\
[-- This %s/%s attachment is not included, --]\n\
[-- and the indicated access-type %s is unsupported --]\n", 199},
  {"Error: multipart/signed has no protocol.", 200},
  {"Error: multipart/encrypted has no protocol parameter!", 201},
  {"Unable to open temporary file!", 202},
  {"[-- %s/%s is unsupported ", 203},
  {"(use '%s' to view this part)", 204},
  {"(need 'view-attachments' bound to key!)", 205},
  {"%s: unable to attach file", 206},
  {"ERROR: please report this bug", 207},
  {"<UNKNOWN>", 208},
  {"\
\n\
Generic bindings:\n\
\n", 209},
  {"\
\n\
Unbound functions:\n\
\n", 210},
  {"Help for %s", 211},
  {"imap_error(): unexpected response in %s: %s\n", 212},
  {"Fatal error.  Message count is out of sync!", 213},
  {"Fetching message headers... [%d/%d]", 214},
  {"Unable to fetch headers from this IMAP server version.", 215},
  {"Reopening mailbox... %s", 216},
  {"This IMAP server is ancient. Mutt does not work with it.", 217},
  {"IMAP Username: ", 218},
  {"Password for %s@%s: ", 219},
  {"Logging in...", 220},
  {"Login failed.", 221},
  {"Selecting %s...", 222},
  {"Unable to append to IMAP mailboxes at this server", 223},
  {"Create %s?", 224},
  {"Fetching message...", 225},
  {"Sending CREATE command ...", 226},
  {"Sending APPEND command ...", 227},
  {"Uploading message ...", 228},
  {"Closing connection to IMAP server...", 229},
  {"Saving message status flags... [%d/%d]", 230},
  {"Expunging messages from server...", 231},
  {"Closing mailbox...", 232},
  {"alias: no address", 233},
  {"invalid header field", 234},
  {"ignoring empty header field: %s", 235},
  {"%s: unknown sorting method", 236},
  {"mutt_restore_default: error in regexp: %s\n", 237},
  {"%s: unknown variable", 238},
  {"prefix is illegal with reset", 239},
  {"value is illegal with reset", 240},
  {"%s is set", 241},
  {"%s is unset", 242},
  {"%s: invalid mailbox type", 243},
  {"%s: invalid value", 244},
  {"%s: unknown type", 245},
  {"Error in %s, line %d: %s", 246},
  {"source: errors in %s", 247},
  {"source: error at %s", 248},
  {"source: too many arguments", 249},
  {"%s: unknown command", 250},
  {"Error in command line: %s\n", 251},
  {"unable to determine home directory", 252},
  {"unable to determine username", 253},
  {"Macro loop detected.", 254},
  {"Key is not bound.", 255},
  {"Key is not bound.  Press '%s' for help.", 256},
  {"push: too many arguments", 257},
  {"%s: no such menu", 258},
  {"null key sequence", 259},
  {"bind: too many arguments", 260},
  {"%s: no such function in map", 261},
  {"macro: empty key sequence", 262},
  {"macro: too many arguments", 263},
  {"exec: too few arguments", 264},
  {"%s: no such command", 265},
  {"null operation", 266},
  {"force viewing of attachment using mailcap", 267},
  {"view attachment as text", 268},
  {"move to the bottom of the page", 269},
  {"remail a message to another user", 270},
  {"select a new file in this directory", 271},
  {"view file", 272},
  {"display the currently selected file's name", 273},
  {"change directories", 274},
  {"check mailboxes for new mail", 275},
  {"attach a file(s) to this message", 276},
  {"attach message(s) to this message", 277},
  {"edit the BCC list", 278},
  {"edit the CC list", 279},
  {"edit attachment description", 280},
  {"edit attachment transfer-encoding", 281},
  {"enter a file to save a copy of this message in", 282},
  {"edit the file to be attached", 283},
  {"edit the from field", 284},
  {"edit the message with headers", 285},
  {"edit the message", 286},
  {"edit attachment using mailcap entry", 287},
  {"edit the Reply-To field", 288},
  {"edit the subject of this message", 289},
  {"edit the TO list", 290},
  {"edit attachment type", 291},
  {"get a temporary copy of an attachment", 292},
  {"run ispell on the message", 293},
  {"compose new attachment using mailcap entry", 294},
  {"save this message to send later", 295},
  {"rename/move an attached file", 296},
  {"send the message", 297},
  {"toggle whether to delete file after sending it", 298},
  {"update an attachment's encoding info", 299},
  {"write the message to a folder", 300},
  {"copy a message to a file/mailbox", 301},
  {"create an alias from a message sender", 302},
  {"move entry to bottom of screen", 303},
  {"move entry to middle of screen", 304},
  {"move entry to top of screen", 305},
  {"make decoded (text/plain) copy", 306},
  {"make decoded copy (text/plain) and delete", 307},
  {"delete the current entry", 308},
  {"delete all messages in subthread", 309},
  {"delete all messages in thread", 310},
  {"display full address of sender", 311},
  {"display message with full headers", 312},
  {"display a message", 313},
  {"edit the current message for resending", 314},
  {"delete the char in front of the cursor", 315},
  {"move the cursor one character to the left", 316},
  {"jump to the beginning of the line", 317},
  {"cycle among incoming mailboxes", 318},
  {"complete filename or alias", 319},
  {"complete address with query", 320},
  {"delete the char under the cursor", 321},
  {"jump to the end of the line", 322},
  {"move the cursor one character to the right", 323},
  {"scroll up through the history list", 324},
  {"delete chars from cursor to end of line", 325},
  {"delete all chars on the line", 326},
  {"delete the word in front of the cursor", 327},
  {"quote the next typed key", 328},
  {"enter a muttrc command", 329},
  {"enter a file mask", 330},
  {"exit this menu", 331},
  {"filter attachment through a shell command", 332},
  {"move to the first entry", 333},
  {"toggle a message's 'important' flag", 334},
  {"forward a message with comments", 335},
  {"select the current entry", 336},
  {"reply to all recipients", 337},
  {"scroll down 1/2 page", 338},
  {"scroll up 1/2 page", 339},
  {"this screen", 340},
  {"jump to an index number", 341},
  {"move to the last entry", 342},
  {"reply to specified mailing list", 343},
  {"execute a macro", 344},
  {"compose a new mail message", 345},
  {"open a different folder", 346},
  {"open a different folder in read only mode", 347},
  {"clear a status flag from a message", 348},
  {"delete messages matching a pattern", 349},
  {"retrieve mail from POP server", 350},
  {"move to the first message", 351},
  {"move to the last message", 352},
  {"show only messages matching a pattern", 353},
  {"jump to the next new message", 354},
  {"jump to the next subthread", 355},
  {"jump to the next thread", 356},
  {"move to the next undeleted message", 357},
  {"jump to the next unread message", 358},
  {"jump to previous thread", 359},
  {"jump to previous subthread", 360},
  {"move to the last undelete message", 361},
  {"jump to the previous new message", 362},
  {"jump to the previous unread message", 363},
  {"mark the current thread as read", 364},
  {"mark the current subthread as read", 365},
  {"set a status flag on a message", 366},
  {"save changes to mailbox", 367},
  {"tag messages matching a pattern", 368},
  {"undelete messages matching a pattern", 369},
  {"untag messages matching a pattern", 370},
  {"move to the middle of the page", 371},
  {"move to the next entry", 372},
  {"scroll down one line", 373},
  {"move to the next page", 374},
  {"jump to the bottom of the message", 375},
  {"return to the main-menu", 376},
  {"toggle display of quoted text", 377},
  {"skip beyond quoted text", 378},
  {"jump to the top of the message", 379},
  {"pipe message/attachment to a shell command", 380},
  {"move to the previous entry", 381},
  {"scroll up one line", 382},
  {"move to the previous page", 383},
  {"print the current entry", 384},
  {"query external program for addresses", 385},
  {"append new query results to current results", 386},
  {"save changes to mailbox and quit", 387},
  {"recall a postponed message", 388},
  {"clear and redraw the screen", 389},
  {"{internal}", 390},
  {"reply to a message", 391},
  {"save message/attachment to a file", 392},
  {"search for a regular expression", 393},
  {"search backwards for a regular expression", 394},
  {"search for next match", 395},
  {"search for next match in opposite direction", 396},
  {"toggle search pattern coloring", 397},
  {"invoke a command in a subshell", 398},
  {"sort messages", 399},
  {"sort messages in reverse order", 400},
  {"tag the current entry", 401},
  {"apply next function to tagged messages", 402},
  {"tag the current subthread", 403},
  {"tag the current thread", 404},
  {"toggle a message's 'new' flag", 405},
  {"toggle whether the mailbox will be rewritten", 406},
  {"toggle whether to browse mailboxes or all files", 407},
  {"move to the top of the page", 408},
  {"undelete the current entry", 409},
  {"undelete all messages in thread", 410},
  {"undelete all messages in subthread", 411},
  {"show the Mutt version number and date", 412},
  {"view attachment using mailcap entry if necessary", 413},
  {"show MIME attachments", 414},
  {"show currently active limit pattern", 415},
  {"collapse/uncollapse current thread", 416},
  {"collapse/uncollapse all threads", 417},
  {"attach a PGP public key", 418},
  {"show PGP options", 419},
  {"extract PGP public keys", 420},
  {"wipe PGP passphrase from memory", 421},
  {"mail a PGP public key", 422},
  {"verify a PGP public key", 423},
  {"view the key's user id", 424},
  {"make decrypted copy and delete", 425},
  {"make decrypted copy", 426},
  {"Out of memory!", 427},
  {"File is a directory, save under it?", 428},
  {"File under directory: ", 429},
  {"File exists, (o)verwrite, (a)ppend, or (c)ancel?", 430},
  {"oac", 431},
  {"%s is not a mailbox!", 432},
  {"Append messages to %s?", 433},
  {"To contact the developers, please mail to <mutt-dev@mutt.org>.\n", 434},
  {"\
Copyright (C) 1996-8 Michael R. Elkins and others.\n\
Mutt comes with ABSOLUTELY NO WARRANTY; for details type `mutt -vv'.\n\
Mutt is free software, and you are welcome to redistribute it\n\
under certain conditions; type `mutt -vv' for details.\n", 435},
  {"\
Copyright (C) 1996-8 Michael R. Elkins <me@cs.hmc.edu>\n\
Copyright (C) 1997-8 Thomas Roessler <roessler@guug.de>\n\
Copyright (C) 1998   Werner Koch <wk@isil.d.shuttle.de>\n\
Copyright (C) 1998   Ruslan Ermilov <ru@ucb.crimea.ua>\n\
\n\
Lots of others not mentioned here contributed lots of code,\n\
fixes, and suggestions.\n\
\n\
    This program is free software; you can redistribute it and/or modify\n\
    it under the terms of the GNU General Public License as published by\n\
    the Free Software Foundation; either version 2 of the License, or\n\
    (at your option) any later version.\n\
\n\
    This program is distributed in the hope that it will be useful,\n\
    but WITHOUT ANY WARRANTY; without even the implied warranty of\n\
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n\
    GNU General Public License for more details.\n\
\n\
    You should have received a copy of the GNU General Public License\n\
    along with this program; if not, write to the Free Software\n\
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.\n", 436},
  {"\
\n\
SHA1 implementation Copyright (C) 1995-7 Eric A. Young <eay@cryptsoft.com>\n\
\n\
    Redistribution and use in source and binary forms, with or without\n\
    modification, are permitted under certain conditions.\n\
\n\
    The SHA1 implementation comes AS IS, and ANY EXPRESS OR IMPLIED\n\
    WARRANTIES, including, but not limited to, the implied warranties of\n\
    merchantability and fitness for a particular purpose ARE DISCLAIMED.\n\
\n\
    You should have received a copy of the full distribution terms\n\
    along with this program; if not, write to the program's developers.\n", 437},
  {"\
usage: mutt [ -nRzZ ] [ -e <cmd> ] [ -F <file> ] [ -m <type> ] [ -f <file> \
]\n\
       mutt [ -nx ] [ -e <cmd> ] [ -a <file> ] [ -F <file> ] [ -H <file> ] [ \
-i <file> ] [ -s <subj> ] [ -b <addr> ] [ -c <addr> ] <addr> [ ... ]\n\
       mutt [ -n ] [ -e <cmd> ] [ -F <file> ] -p\n\
       mutt -v[v]\n\
\n\
options:\n\
  -a <file>\tattach a file to the message\n\
  -b <address>\tspecify a blind carbon-copy (BCC) address\n\
  -c <address>\tspecify a carbon-copy (CC) address\n\
  -e <command>\tspecify a command to be executed after initialization\n\
  -f <file>\tspecify which mailbox to read\n\
  -F <file>\tspecify an alternate muttrc file\n\
  -H <file>\tspecify a draft file to read header from\n\
  -i <file>\tspecify a file which Mutt should include in the reply\n\
  -m <type>\tspecify a default mailbox type\n\
  -n\t\tcauses Mutt not to read the system Muttrc\n\
  -p\t\trecall a postponed message\n\
  -R\t\topen mailbox in read-only mode\n\
  -s <subj>\tspecify a subject (must be in quotes if it has spaces)\n\
  -v\t\tshow version and compile-time definitions\n\
  -x\t\tsimulate the mailx send mode\n\
  -y\t\tselect a mailbox specified in your `mailboxes' list\n\
  -z\t\texit immediately if there are no messages in the mailbox\n\
  -Z\t\topen the first folder with new message, exit immediately if none\n\
  -h\t\tthis help message", 438},
  {"\
\n\
Compile options:", 439},
  {"Error initializing terminal.", 440},
  {"Debugging at level %d.\n", 441},
  {"DEBUG was not defined during compilation.  Ignored.\n", 442},
  {"No recipients specified.\n", 443},
  {"%s: unable to attach file.\n", 444},
  {"No mailbox with new mail.", 445},
  {"Mailbox is empty.", 446},
  {"Mailbox is corrupt!", 447},
  {"Reading %s... %d (%d%%)", 448},
  {"Mailbox was corrupted!", 449},
  {"Fatal error!  Could not reopen mailbox!", 450},
  {"Unable to lock mailbox!", 451},
  {"sync: mbox modified, but no modified messages! (report this bug)", 452},
  {"Writing messages... %d (%d%%)", 453},
  {"Write failed!  Saved partial mailbox to %s", 454},
  {"Could not reopen mailbox!", 455},
  {"Reopening mailbox...", 456},
  {"Jump to: ", 457},
  {"Invalid index number.", 458},
  {"No entries.", 459},
  {"You cannot scroll down farther.", 460},
  {"You cannot scroll up farther.", 461},
  {"You are on the last page.", 462},
  {"You are on the first page.", 463},
  {"First entry is shown.", 464},
  {"Last entry is shown.", 465},
  {"You are on the last entry.", 466},
  {"You are on the first entry.", 467},
  {"Search for: ", 468},
  {"Reverse search for: ", 469},
  {"No search pattern.", 470},
  {"Not found.", 471},
  {"No tagged entries.", 472},
  {"Search is not implemented for this menu.", 473},
  {"Tagging is not supported.", 474},
  {"Reading %s... %d", 475},
  {"Lock count exceeded, remove lock for %s?", 476},
  {"Can't dotlock %s.\n", 477},
  {"Timeout exceeded while attempting fcntl lock!", 478},
  {"Waiting for fcntl lock... %d", 479},
  {"Timeout exceeded while attempting flock lock!", 480},
  {"Waiting for flock attempt... %d", 481},
  {"Reading %s...", 482},
  {"Writing %s...", 483},
  {"Could not synchronize mailbox %s!", 484},
  {"Move read messages to %s?", 485},
  {"Purge %d deleted message?", 486},
  {"Purge %d deleted messages?", 487},
  {"Moving read messages to %s...", 488},
  {"Mailbox is unchanged.", 489},
  {"%d kept, %d moved, %d deleted.", 490},
  {"%d kept, %d deleted.", 491},
  {" Press '%s' to toggle write", 492},
  {"Use 'toggle-write' to re-enable write!", 493},
  {"Mailbox is marked unwritable. %s", 494},
  {"Not available in this menu.", 495},
  {"PrevPg", 496},
  {"NextPg", 497},
  {"View Attachm.", 498},
  {"Next", 499},
  {"Bottom of message is shown.", 500},
  {"Top of message is shown.", 501},
  {"Reverse search: ", 502},
  {"Search: ", 503},
  {"Help is currently being shown.", 504},
  {"No more quoted text.", 505},
  {"No more unquoted text after quoted text.", 506},
  {"multipart message has no boundary parameter!", 507},
  {"Error in expression: %s", 508},
  {"Invalid day of month: %s", 509},
  {"Invalid month: %s", 510},
  {"error in expression", 511},
  {"error in pattern at: %s", 512},
  {"%c: invalid command", 513},
  {"%c: not supported in this mode", 514},
  {"missing parameter", 515},
  {"mismatched parenthesis: %s", 516},
  {"empty pattern", 517},
  {"error: unknown op %d (report this error).", 518},
  {"Compiling search pattern...", 519},
  {"Executing command on matching messages...", 520},
  {"No messages matched criteria.", 521},
  {"Search hit bottom without finding match", 522},
  {"Search hit top without finding match", 523},
  {"Search interrupted.", 524},
  {"Enter PGP passphrase:", 525},
  {"PGP passphrase forgotten.", 526},
  {"Unknown PGP version \"%s\".", 527},
  {"[-- PGP output follows (current time: ", 528},
  {"[-- Error: unable to create PGP subprocess! --]\n", 529},
  {"\
\n\
[-- End of PGP output --]\n\
\n", 530},
  {"\
[-- BEGIN PGP MESSAGE --]\n\
\n", 531},
  {"[-- BEGIN PGP PUBLIC KEY BLOCK --]\n", 532},
  {"\
[-- BEGIN PGP SIGNED MESSAGE --]\n\
\n", 533},
  {"\
\n\
[-- END PGP MESSAGE --]\n", 534},
  {"[-- END PGP PUBLIC KEY BLOCK --]\n", 535},
  {"\
\n\
[-- END PGP SIGNED MESSAGE --]\n", 536},
  {"\
[-- Error: could not find beginning of PGP message! --]\n\
\n", 537},
  {"\
[-- End of PGP output --]\n\
\n", 538},
  {"\
[-- The following data is PGP/MIME signed --]\n\
\n", 539},
  {"\
\n\
[-- End of PGP/MIME signed data --]\n", 540},
  {"\
[-- Error: this message does not comply with the PGP/MIME specification! \
--]\n\
\n", 541},
  {"Internal error. Inform <roessler@guug.de>.", 542},
  {"\
[-- Error: could not create a PGP subprocess! --]\n\
\n", 543},
  {"\
[-- Error: malformed PGP/MIME message! --]\n\
\n", 544},
  {"[-- Error: could not create temporary file! --]\n", 545},
  {"\
[-- The following data is PGP/MIME encrypted --]\n\
\n", 546},
  {"\
\n\
[-- End of PGP/MIME encrypted data --]\n", 547},
  {"Can't open PGP subprocess!", 548},
  {"Use keyID = \"%s\" for %s?", 549},
  {"Enter keyID for %s: ", 550},
  {"Exit  ", 551},
  {"Select  ", 552},
  {"Check key  ", 553},
  {"PGP keys matching ", 554},
  {"Can't open /dev/null", 555},
  {"Can't create temporary file", 556},
  {"Can't create filter", 557},
  {"Key ID: 0x%s", 558},
  {"This ID's trust level is undefined.", 559},
  {"This ID is not trusted.", 560},
  {"This ID is only marginally trusted.", 561},
  {"%s Do you really want to use it?", 562},
  {"Please enter the key ID: ", 563},
  {"PGP Key 0x%s.", 564},
  {"reserved", 565},
  {"Encrypted Session Key", 566},
  {"Signature Packet", 567},
  {"Conventionally Encrypted Session Key Packet", 568},
  {"One-Pass Signature Packet", 569},
  {"Secret Key Packet", 570},
  {"Public Key Packet", 571},
  {"Secret Subkey Packet", 572},
  {"Compressed Data Packet", 573},
  {"Symmetrically Encrypted Data Packet", 574},
  {"Marker Packet", 575},
  {"Literal Data Packet", 576},
  {"Trust Packet", 577},
  {"Name Packet", 578},
  {"Subkey Packet", 579},
  {"Reserved", 580},
  {"Comment Packet", 581},
  {"POP Password: ", 582},
  {"POP host is not defined.", 583},
  {"No POP username is defined.", 584},
  {"Could not find address for host %s.", 585},
  {"Connecting to %s", 586},
  {"Server closed connection!", 587},
  {"No new mail in POP mailbox.", 588},
  {"Reading %d new message (%d bytes)...", 589},
  {"Reading %d new messages (%d bytes)...", 590},
  {"Error reading message!", 591},
  {"Error while writing mailbox!", 592},
  {"%s [%d messages read]", 593},
  {"Postponed Messages", 594},
  {"No postponed messages.", 595},
  {"Illegal PGP header", 596},
  {"New Query", 597},
  {"Make Alias", 598},
  {"Search", 599},
  {"Waiting for response...", 600},
  {"Query command not defined.", 601},
  {"Query", 602},
  {"Query: ", 603},
  {"Query '%s'", 604},
  {"Pipe", 605},
  {"Print", 606},
  {"Saving...", 607},
  {"Attachment saved", 608},
  {"WARNING!  You are about to overwrite %s, continue?", 609},
  {"Attachment filtered.", 610},
  {"Filter through: ", 611},
  {"Pipe to: ", 612},
  {"I dont know how to print %s attachments!", 613},
  {"Print tagged attachment(s)?", 614},
  {"Print attachment?", 615},
  {"You may only bounce message/rfc822 parts.", 616},
  {"Bounce messages to %s...?", 617},
  {"Bounce message to %s...?", 618},
  {"Attachments", 619},
  {"Deletion of attachments from PGP messages is unsupported.", 620},
  {"Only deletion of multipart attachments is supported.", 621},
  {"This operation is not currently supported for PGP messages.", 622},
  {"Improperly formated entry for type %s in \"%s\" line %d", 623},
  {"No mailcap path specified", 624},
  {"mailcap entry for type %s not found", 625},
  {"score: too few arguments", 626},
  {"score: too many arguments", 627},
  {"No subject, abort?", 628},
  {"No subject, aborting.", 629},
  {"Reply to %s%s?", 630},
  {"No tagged messages are visible!", 631},
  {"No mailing lists found!", 632},
  {"Include message in reply?", 633},
  {"Could not include all requested messages!", 634},
  {"Forward MIME encapsulated?", 635},
  {"Recall postponed message?", 636},
  {"Abort unmodified message?", 637},
  {"Aborted unmodified message.", 638},
  {"Mail not sent.", 639},
  {"Message postponed.", 640},
  {"No recipients are specified!", 641},
  {"No recipients were specified.", 642},
  {"No subject, abort sending?", 643},
  {"No subject specified.", 644},
  {"Message edited. Really send?", 645},
  {"Sending message...", 646},
  {"Could not send the message.", 647},
  {"Mail sent.", 648},
  {"Sending in background.", 649},
  {"No boundary parameter found! [report this error]", 650},
  {"%s no longer exists!", 651},
  {"Could not open %s", 652},
  {"Error sending message, child exited %d (%s).", 653},
  {"Output of the delivery process", 654},
  {"%s...  Exiting.\n", 655},
  {"Caught %s...  Exiting.\n", 656},
  {"Caught signal %d...  Exiting.\n", 657},
  {"Connecting to %s...", 658},
  {"Sorting mailbox...", 659},
  {"Could not find sorting function! [report this bug]", 660},
  {"(no mailbox)", 661},
};

int _msg_tbl_length = 661;
